
/*
 * The mask is be:
 * GPC_AUTODETECT_MASK_PARPORT parport selected
 * GPC_AUTODETECT_MASK_NETWORK network selected
 * GPC_AUTODETECT_MASK_USB usb selected
 * 
 * Or these together to represent multiple selections.
 * 
 * The data will be a vector of all the text fields for each of
 * the detection methods.
 * 
 * data[GPC_AUTODETECT_OFF_PARPORT] the text field after the parport
 * data[GPC_AUTODETECT_OFF_NETWORK] the text field after the network
 * data[GPC_AUTODETECT_OFF_USB] the text field after the usb
 *
 * Return value is a vector of strings representing detected printers
 * (terminated by a NULL).
 */
#include <stdlib.h>

#include <glib.h>

#include "gpc_autodetect.h"
#include "../printerconf/src/printerconf.h"

static char **merge_lists(char **dest, char **src)
{
  char **cur = NULL;
  int len_src = 0;
  int len_dest = 0;

  for(cur = src; *cur; cur++)
  {}
  len_src = cur - src;

  for(cur = dest; *cur; cur++)
  {}
  len_src = cur - dest;

  dest = realloc(dest, sizeof(char *) * (len_src + len_dest + 1));
  if (!dest) abort();

  cur = dest + len_dest;
  memcpy(cur, src, (len_src + 1) * sizeof(char *));
  return dest;
}


char **gpc_autodetect(int mask, const char **data)
{
  char **ret = NULL;
  char **prns=NULL;
  int lens;

  // they didn't select anything;
  if(mask==0){
    ret = malloc(sizeof(char *));
    ret[0] = NULL;
    return (ret);
  }

  if((mask&GPC_AUTODETECT_MASK_PARPORT)!=0){
    ret=pconf_detect_printer(PCONF_DETECT_PPORT,data[0],&lens);
  }
  if((mask&GPC_AUTODETECT_MASK_NETWORK)!=0){
    prns=pconf_detect_printer(PCONF_DETECT_NETWORK,data[1],&lens);
    if(ret==NULL)
      ret=prns;
    else
      ret=merge_lists(ret,prns);
  }
  if((mask&GPC_AUTODETECT_MASK_USB)!=0){
    prns=pconf_detect_printer(PCONF_DETECT_USB,data[2],&lens);
    if(ret==NULL)
      ret=prns;
    else
      ret=merge_lists(ret,prns);
  }
  return ret;
}
