#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdarg.h>

#include <gnome.h>

#include "gpc_pdinfo.h"
#include "gpc_support.h"
#include "gpc_queue.h"

static int gpc_show_version = FALSE;

static char *gpc_printcap_file = NULL;

static int gpc_do_add_printer = FALSE;
static int gpc_do_del_printer = FALSE;
static int gpc_do_edt_printer = FALSE;

static char *gpc_queue_name = NULL;
static char *gpc_printer_type = NULL;

void gpc_cmd_line_main(int argc, char *argv[]);

void gpc_cmd_line_main(int argc, char *argv[])
{
  const struct poptOption gpc_popt_options [] =
    {
     {"version", 'v', POPT_ARG_NONE, &gpc_show_version, 0,
      N_("Display GPC's version"), NULL},

     {"printcap-file", 'f', POPT_ARG_STRING, &gpc_printcap_file, 0,
      N_("The file to use as the printcap database"), NULL},

     
     {"add", 0, POPT_ARG_NONE, &gpc_do_add_printer, 0,
      N_("Add a printer to the database"), NULL},
     {"delete", 0, POPT_ARG_NONE, &gpc_do_del_printer, 0,
      N_("Delete a printer from the database"), NULL},
     {"edit", 0, POPT_ARG_NONE, &gpc_do_edt_printer, 0,
      N_("Edit a printer in the database"), NULL},
     
     {"queue-name", 0, POPT_ARG_STRING, &gpc_queue_name, 0,
      N_("Set the name of the printer queue"), N_("NAME")},
     {"type", 0, POPT_ARG_STRING, &gpc_printer_type, 0,
      N_("Set the type of the printer queue (Eg. local or remote)"), N_("TYPE")},
     
     { NULL, '\0', 0, NULL, 0,NULL,NULL }
    };
  poptContext ctx;
  GPC_QUEUE_INFO *qinfo = NULL;
  
  gnome_init_with_popt_table("gpc", VERSION, argc, argv, gpc_popt_options,
                             0, &ctx);

  if (gpc_printcap_file)
    gpc_queue_get_list(gpc_printcap_file);
  
  if ((gpc_do_add_printer && gpc_do_del_printer) ||
      (gpc_do_add_printer && gpc_do_edt_printer) ||
      (gpc_do_del_printer && gpc_do_edt_printer))
  {
    fprintf(stderr, " You can only do one of add, delete or edit.\n");
    exit (EXIT_FAILURE);
  }
  
  if (gpc_show_version)
  {
    printf (_("GPC version '%s'\n"), VERSION);
    exit (EXIT_SUCCESS);
  }

  if ((gpc_do_add_printer || gpc_do_del_printer || gpc_do_edt_printer) &&
      !gpc_queue_name)
  {
    fprintf(stderr, " There is no specified queue name.\n");
    exit (EXIT_FAILURE);
  }

  if (gpc_do_add_printer || gpc_do_edt_printer)
  {
    qinfo = g_malloc0(sizeof(GPC_QUEUE_INFO));

    if (FALSE)
    { }
    else if (!strcasecmp(gpc_printer_type, "local"))
      qinfo->qtype = GPC_QTYPE_LOCAL;
    else if (!strcasecmp(gpc_printer_type, "remote"))
      qinfo->qtype = GPC_QTYPE_REMOTE;
    else if (!strcasecmp(gpc_printer_type, "direct"))
      qinfo->qtype = GPC_QTYPE_DIRECT;
    else if (!strcasecmp(gpc_printer_type, "smb") ||
             !strcasecmp(gpc_printer_type, "samba") ||
             FALSE)
      qinfo->qtype = GPC_QTYPE_SMB;
    else if (!strcasecmp(gpc_printer_type, "ncp") ||
             !strcasecmp(gpc_printer_type, "netware") ||
             FALSE)
      qinfo->qtype = GPC_QTYPE_NCP;
    else
    {
      fprintf(stderr, " Invalid printer type \"%s\".\n", gpc_printer_type);
      exit (EXIT_FAILURE);
    }

    qinfo->qlist = g_malloc(sizeof(char *) * 2);
    qinfo->qlist[0] = g_strdup(gpc_queue_name);
    qinfo->qlist[1] = NULL;

    /* FIXME: add more stuff.... */
  }
  
  if (gpc_do_add_printer)
  {    
    gpc_queue_add(gpc_queue_get_list(NULL), qinfo); // FIXME JAMES
    exit (EXIT_SUCCESS);
  }
  if (gpc_do_del_printer)
  {
    gpc_queue_del(gpc_queue_get_list(NULL), gpc_queue_name); // FIXME JAMES
    exit (EXIT_SUCCESS);
  }
  if (gpc_do_edt_printer)
  {
    gpc_queue_edt(gpc_queue_get_list(NULL), qinfo); // FIXME JAMES
    exit (EXIT_SUCCESS);
  }
  
  if (ctx)
  { /* has xtra arguments */; }
  
}
