#include <gnome.h>

#include "gpc_pdinfo.h"
#include "gpc_support.h"
#include "gpc_queue.h"
#include "support.h"

/*----------------------------------------------------------------
  Check if the widget is of the specified GTK type.
----------------------------------------------------------------*/
int widget_is_type (GtkWidget *widget, const char *wtype) {
  return gtk_type_is_a (GTK_OBJECT_TYPE(widget), gtk_type_from_name (wtype));
}

/*----------------------------------------------------------------
  Given any widget, a font name, and a null-terminated list of
  widget name strings, change each of those named widgets to the
  specified font.
----------------------------------------------------------------*/
int set_widgets_font (GtkWidget *widget_in, const char *font_name, ...) {
  GdkFont *font;
  GtkStyle *style;
  va_list argp;
  char *widget_name = NULL;
  GtkWidget *widget;

  va_start (argp, font_name);

  /* Step through list of widgets */
  widget_name = va_arg(argp, char *);

  while (widget_name) {
    widget = lookup_widget (GTK_WIDGET (widget_in), widget_name);

    /* Get copy of the widget's style, and change 
       the font within that style. */
    style = gtk_style_copy (widget->style);

    if (style->font)
      gdk_font_unref (style->font);

    font = gdk_fontset_load(font_name);
    style->font = font;

    if (widget_is_type (widget, "GtkButton"))
      gtk_widget_set_style(GTK_WIDGET(GTK_BIN(widget)->child), style);
    else
      gtk_widget_set_style(GTK_WIDGET(widget), style);

    widget_name = va_arg(argp, char *);
  }

  va_end (argp);
  return 0;
}

/*----------------------------------------------------------------
  Remove all leading and trailing whitespace ([\n\r\t ]) from str,
  in place.  Return pointer to chopped char string (str).
----------------------------------------------------------------*/
char * chop (char * str) {
  int front;
  int cur;

  if (str == NULL) return str;

  /* Find first non-blank character. */
  for (cur=0; strchr(" \r\n\t", str[cur]) && (str[cur] != '\0'); cur++); 

  /* Shift string to left. */
  for (front=0; str[cur] != '\0'; cur++) 
    str[front++] = str[cur];

  str[front] = '\0';

  /* Trim the right end of the string. */
  for (cur=strlen(str)-1; cur > 0; cur--)
    if (strchr (" \r\n\t", str[cur]))  
      str[cur] = '\0';
    else
      break;

  return str;
}

/*----------------------------------------------------------------
----------------------------------------------------------------*/
gboolean manuf_model_selected( GtkWidget *widget)
{
  GtkCList *clist = NULL;
  char *manuf = NULL;
  char *model = NULL;

  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (widget), "cltManuf"));
  manuf = clist_get_first_selected_text (GTK_CLIST (clist));
  
  clist = GTK_CLIST (lookup_widget (GTK_WIDGET (widget), "cltPrinterModel")); 
  model = clist_get_first_selected_text (GTK_CLIST (clist));
  
  return (manuf && model);
}

/*----------------------------------------------------------------
----------------------------------------------------------------*/
gboolean auto_model_selected(GtkWidget *widget)
{
  GtkCList *clist = NULL;
  char *model = NULL;

  clist = GTK_CLIST(lookup_widget(GTK_WIDGET(widget), "clist1"));
  model = clist_get_first_selected_text(GTK_CLIST(clist));
  
  return (model && TRUE);
}

/*----------------------------------------------------------------
----------------------------------------------------------------*/
char *clist_get_first_selected_text( GtkCList *clist) {
  char *text = NULL;
  int row = -1;

  if (clist->selection) {
    row = GPOINTER_TO_INT (clist->selection->data);
    if (row >= 0)
      gtk_clist_get_text (clist, row, 0, (gchar **) &text);
  }
  return text;
}

/*----------------------------------------------------------------
----------------------------------------------------------------*/
void set_pixmap_xpm(GtkWidget *widget, gchar ** xpm_data) {
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;

  colormap = gtk_widget_get_colormap(widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d
    (NULL, colormap, &mask, NULL, xpm_data);
  
  if (gdkpixmap == NULL)
    g_error("Couldn't create pixmap from data.");
  gtk_pixmap_set (GTK_PIXMAP(widget), gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
}

/*----------------------------------------------------------------
----------------------------------------------------------------*/
void charlist_to_clist (char **list, GtkWidget *widget) {
  int numitems;
  gchar *data[1];

  numitems = 0;
  if (list)
    while (list[numitems]) {
      data[0] = g_strdup (list[numitems]);
      gtk_clist_append (GTK_CLIST(widget), data);
      g_free (data[0]);
      numitems++;
    }

}

/*----------------------------------------------------------------
----------------------------------------------------------------*/
void pdinfo_to_clist ( GPC_PDINFO **list, GtkWidget *widget) {
  int numitems;
  gchar *data[1];

  numitems = 0;
  if (list)
    while (list[numitems]) {
      data[0] = g_strdup (list[numitems]->brief_desc);
      gtk_clist_append (GTK_CLIST(widget), data);
      free (data[0]);
      numitems++;
    }

}

/*----------------------------------------------------------------
----------------------------------------------------------------*/
void setup_clist_from_qlist(GtkWidget *clist)
{
  // FIXME james you will need to fill this in I don't know
  // where to get the data from.
  GPC_QUEUE_LIST **qlist = gpc_queue_get_list(NULL);
  GPC_QUEUE_LIST **scan = qlist;

  gtk_clist_freeze(GTK_CLIST(clist));
  
  while (*scan)
  {
    char *a[2];
    
    a[0] = (*scan)->qname;
    a[1] = NULL;
    
    gtk_clist_append(GTK_CLIST(clist), a);
    
    ++scan;
  }

  gtk_clist_thaw(GTK_CLIST(clist));
}
